const svg = d3.select('svg'); //selecting the SVG element from d3.js resources
const projection = d3.geoNaturalEarth1(); //assigning the geoNaturalEarth1() projection
let cafrica = 0;
//getting the json data from the link provided
d3.json(
  'https://gist.githubusercontent.com/prachow/ea0f513115ceab3b380b3eff573eed46/raw/94f2f899e6f4f3c5b886cfbffed0ce1c2f9dc08a/Africa.csv',
  function (json) {
    const presimplify = topojson.presimplify(
      json
    ); //performing the simplification process to have efficient map
    const geojson = topojson.feature(
      presimplify,
      presimplify.objects
        .continent_Africa_subunits
    ); //converting the topojson data into geojson data
  
    draw(geojson, cafrica);  //Implementing the draw method
    const reDraw = draw.bind(null, geojson);//Implementing the redraw function with functionality as draw function
//implementing the window resize so that the map exactly fits in the presented window
    d3.select(window).on('resize', function () {
      reDraw(cafrica);
    });
  }
);

//Adjusting the window formats so that the map is clearly visible when redeaws it
function draw(geojson, threshold) {
  const w = svg.node().clientWidth;
  const h = svg.node().clientHeight;

  projection.fitExtent(
    [
      [0, 0],
      [w, h],
    ],
    geojson
  );

  //Rendering the json data 
  const simplify = d3.geoTransform({
    point: function (a, b, c) {
      //converting into the pixel co-ordinates using projection function
      if (c >= threshold)
        this.stream.point(projection([a, b]));
    },
  });

  //setting the path to the SVG elements to visualize the Africa map
  const path = d3.geoPath().projection(simplify);
  //binding the data to svg to visualize the updated data 
  const updateMaps = svg
    .selectAll('.map')
    .data([geojson]);
  const enterMaps = updateMaps
    .enter()
    .append('path')
    .attr('class', 'map');

  updateMaps.merge(enterMaps).attr('d', path);//updating the map on web page for every change in geojson data
}
